﻿/*  www.vb-paradise.de/index.php/User/23082-exc-jdbi/
 *  github.com/exc-jdbi
 *  Is Free and Opensource
*/
using System;
using System.Text;
using System.Numerics;
using System.Collections.Generic;
using System.Security.Cryptography;
using System.Runtime.InteropServices;
namespace ModifiedRNGCryptoServiceProvider
{
    /// <summary>
    /// Modified RNGCryptoServiceProvider
    /// </summary>
    /// <history>© exc-jdbi 2017</history>
    [System.Diagnostics.DebuggerStepThrough()]
    public class CryptoRnd
    {
        private Int64 _min;
        private UInt64 _max;
        private double _maxf = 0.0, _minf = double.MaxValue;
        private RNGCryptoServiceProvider rng = new RNGCryptoServiceProvider();

        #region "Secure Randoms"
        /// <summary>
        /// Set Random Area - Interval min / max. 
        /// </summary>
        /// <param name="min">Minimum</param>
        /// <param name="max">Maximum</param>
        /// <returns>Return True if min / max correctly</returns>
        public bool Interval<T>(T min, T max)
            where T : IComparable
        {
            if (max.CompareTo(min) > 0)
            {
                this._min = TryChangeType<Int64, T>(min);
                this._max = TryChangeType<UInt64, T>(max);
                return true;
            }
            return false;
        }
        /// <summary>
        /// Return a Array with Randoms as T
        /// </summary>
        /// <param name="arr">Return a Array with Randoms</param>
        public void GetRnds<T>(ref T[] arr)
            where T : IComparable
        {
            if ((arr != null) && (arr.Length > 0))
                for (Int32 i = 0; i < arr.Length; i++)
                    arr[i] = this.Next<T>();
        }
        /// <summary>
        /// Return a Array with Randoms  as T in Interval min / max. 
        /// </summary>
        /// <param name="arr">Return a Array with Randoms</param>
        /// <param name="min">Minimum</param>
        /// <param name="max">Maximum</param>
        public void GetRnds<T>(ref T[] arr, T min, T max)
            where T : IComparable
        {
            if ((arr != null) && (arr.Length > 0))
                if (this.Interval<T>(min, max))
                    this.GetRnds<T>(ref arr);
        }
        /// <summary>
        /// Next Random as T
        /// </summary>  
        /// <param name="min">Minimum</param>
        /// <param name="max">Maximum</param>
        /// <returns>Returns a Number as T</returns>
        public T Next<T>(T min, T max)
            where T : IComparable
        {
            if (this.Interval<T>(min, max))
                return this.Next<T>();
            return default(T);
        }
        /// <summary>
        /// Next Secure-Random as T
        /// </summary>
        /// <returns>Returns a Secure-Number as T</returns>
        public T Next<T>() where T : IComparable
        {
            Int32 decCount = 0;
            T res = default(T);
            BigInteger n = default(BigInteger);
            decimal r = default(decimal);

            //Calc Difference
            n = new BigInteger(this._max) - new BigInteger(this._min);
            r = (decimal)this.nextRnd();

            //Calculate the number of decimal places
            decCount = GetDecCount(r);

            //Accurate calculation with biginteger
            UInt64 expDec = Convert.ToUInt64(Math.Pow(10, decCount));
            BigInteger tt = new BigInteger(r * expDec);

            //And divide again
            tt *= n; tt /= expDec;

            BigInteger bi2 = new BigInteger(this._min);
            if (tt >= n)
                res = TryChangeType2<T, UInt64>(this._max);
            else res = TryChangeType2<T, Bignum>(new Bignum(bi2 + tt));
            return res;
        }

        #endregion
        #region "Secure Random Primes"
        /// <summary>
        /// Next Secure-RandomPrime as T
        /// </summary>
        /// <returns>Returns a Secure-Prime as T</returns>
        public T NextPrime<T>() where T : IComparable
        {
            if ((_min >= 0) && (_max >= 0))
                return this.FindPrime<T>();
            return default(T);
        }
        /// <summary>
        /// Next Secure-RandomPrime as T
        /// </summary>
        /// <param name="min">Minimum</param>
        /// <param name="max">Maximum</param>
        /// <returns>Returns a Secure-Prime as T</returns>
        public T NextPrime<T>(T min, T max) where T : IComparable
        {
            if (this.Interval<T>(min, max))
                if ((_min >= 0) && (_max >= 0))
                    return this.FindPrime<T>();
            return default(T);
        }
        /// <summary>
        /// Return a Array with Secure-RandomsPrimes as T
        /// </summary>
        /// <param name="arr">Return a Array with Secure-RandomsPrimes</param>
        public void GetRndPrimes<T>(ref T[] arr) where T : IComparable
        {
            if ((_min >= 0) && (_max >= 0))
                if ((arr != null) && (arr.Length > 0))
                    for (Int32 i = 0; i <= arr.Length - 1; i++)
                        arr[i] = this.FindPrime<T>();
        }
        /// <summary>
        /// Return a Array with Secure-RandomsPrimes as T in Interval min / max. 
        /// </summary>
        /// <param name="min">Minimum</param>
        /// <param name="max">Maximum</param>
        /// <param name="arr">Return a Array with Secure-RandomsPrimes</param>
        public void GetRndPrimes<T>(ref T[] arr, T min, T max) where T : IComparable
        {
            if (this.Interval<T>(min, max))
                this.GetRndPrimes<T>(ref arr);
        }
        #endregion
        #region "Secure Random Double"
        public double NextDbl()
        {
            return this.nextRnd();
            //return this.GetRnd() * (1.0D / (double)(UInt64.MaxValue));
        }
        /// <summary>
        /// Return a Array with Randoms as Double less 1.0
        /// </summary>
        public void GetRndsDbl(ref double[] arr)
        {
            if ((arr != null) && (arr.Length > 0))
                for (Int32 i = 0; i < arr.Length; i++)
                    arr[i] = this.NextDbl();
        }
        #endregion
        #region "Secure Random String"
        /// <summary>
        /// Returns a String with Randoms Length/Char alphanumeric
        /// </summary>
        /// <returns>Returns a RandomString alphanumeric</returns>
        public string GetRndAlpha()
        {
            Int32 max = Convert.ToInt32(Math.Pow(2, 8));
            Int32 len = this.Next<Int32>(8, max);
            return GetRndAlpha(len);
        }
        /// <summary>
        /// Returns a String with Randoms as Char alphanumeric
        /// </summary>
        /// <param name="len">String.length</param>
        /// <returns>Returns a RandomString alphanumeric</returns>
        public string GetRndAlpha(Int32 len)
        {
            if (len > 7)
            {
                byte[] b = new byte[len];
                byte[] alpha = AlphaString();
                for (Int32 i = 0; i < len; i++)
                    b[i] = alpha[this.Next(0, alpha.Length - 1)];
                if (b.Length > 0)
                    return Encoding.Default.GetString(b);
            }
            return string.Empty;
        }
        /// <summary>
        /// Returns a String with Randoms Length/Char asciinumeric
        /// </summary>
        /// <returns>Returns a RandomString asciinumeric</returns>
        public string GetRndAscii()
        {
            Int32 max = (Int32)(Math.Pow(2, 8));
            Int32 len = this.Next<Int32>(8, max);
            return GetRndAscii(len);
        }
        /// <summary>
        /// Returns a String with Randoms as Char asciinumeric
        /// </summary>
        /// <param name="len">String.length</param>
        /// <returns>Returns a RandomString asciinumeric</returns>
        public string GetRndAscii(Int32 len)
        {
            if (len > 7)
            {
                byte[] b = new byte[len];
                for (Int32 i = 0; i < len; i++)
                    b[i] = this.Next<byte>(33, 255);
                if (b.Length > 0)
                    return Encoding.Default.GetString(b);
            }
            return string.Empty;
        }
        #endregion
        #region "Methodes"
        [System.Diagnostics.DebuggerHidden()]
        private Int32 GetDecCount(decimal dec)
        {
            Int32 res = (Decimal.GetBits(dec)[3] & 0xff0000) >> 0x10;
            if (res > 19) res = 19;
            return res;
        }
        [System.Diagnostics.DebuggerHidden()]
        private double nextRnd()
        {
            double tmp = this.GetRnd() * (double)(1.0D / (double)(UInt64.MaxValue));
            if (tmp < this._minf) this._minf = tmp;
            if (tmp > this._maxf) this._maxf = tmp;

            if (this._maxf - this._minf == 0.0) this.nextRnd();
            return 1.0 - ((this._maxf - tmp) / (this._maxf - this._minf));
        }
        [System.Diagnostics.DebuggerHidden()]
        private double GetRnd()
        {
            UInt64 scale = 0ul;
            Int32 size = Marshal.SizeOf(typeof(UInt64));
            byte[] buffer = new byte[size];
            this.rng.GetBytes(buffer);

            scale = BitConverter.ToUInt64(buffer, 0);
            return (double)(scale);
        }
        [System.Diagnostics.DebuggerHidden()]
        private byte[] AlphaString()
        {
            List<byte> b = new List<byte>(62);
            byte[] z = { 48, 57, 65, 90, 97, 122 };
            for (Int32 i = 0; i < z.Length; i += 2)
                for (byte j = z[i]; j <= z[i + 1]; j++)
                    b.Add(j);
            if (b.Count > 0)
                return b.ToArray();
            return null;
        }
        [System.Diagnostics.DebuggerHidden()]
        private UInt64 NextUni()
        {
            UInt64 res = 0;
            double n, r, t;
            n = (double)(_max - (UInt64)_min);
            r = nextRnd();
            t = r * n;
            UInt64 b = _max - (UInt64)_min;
            if (t >= b) res = _max;
            else res = (UInt64)_min + (UInt64)(t > b ? b : t);
            return res;
        }
        [System.Diagnostics.DebuggerHidden()]
        private T FindPrime<T>() where T : IComparable
        {
            UInt64 candidate = 0;
            bool bForwart = true;
            while (true)
            {
                candidate = NextUni();
                if (candidate < 3) continue;
                if (candidate % 2uL == 0uL) candidate -= 1uL;
                while (true)
                {
                    if ((candidate >= (UInt64)_min) && (candidate <= _max))
                        if ((candidate <= 5ul) || ((candidate > 5uL) && ((candidate % 5uL) != 0uL)))
                            if (BigInteger.ModPow(2, candidate, candidate) == 2)
                                if (isPrimeMR(candidate))
                                    return TryChangeType2<T, UInt64>(candidate);

                    if (candidate > _max - 2uL) bForwart = false;
                    if (candidate < (UInt64)(_min + 2L)) bForwart = true;
                    if (bForwart) candidate += 2uL;
                    else candidate -= 2uL;
                }
            }
        }
        [System.Diagnostics.DebuggerHidden()]
        private bool isPrimeMR(BigInteger candidate)
        {
            Int32 s = 0;
            BigInteger d = candidate - 1;
            //if ((candidate < 2) || (candidate & 1)==0) return false; 
            while ((d & 1) == 0)
                {d >>= 1; s += 1;}
            BigInteger k = default(BigInteger);
            for (Int32 i = 2; i <= 5; i++)
            {
                k = BigInteger.ModPow(i, d, candidate);
                if ((k == 1) || (k == candidate - 1)) continue;
                for (Int32 j = 1; j < s; j++)
                {
                    k = BigInteger.ModPow(k, 2, candidate);
                    if (k == 1) return false;
                    if (k == candidate - 1) break;
                }
                if (k != candidate - 1) return false;
            }
            return true;
        }
        [System.Diagnostics.DebuggerHidden()]
        private U TryChangeType2<U, V>(V val)
            where V : IConvertible
        {
            try
            {
                Type typ1 = typeof(V), typ2 = typeof(U);
                return (U)Convert.ChangeType(Convert.ChangeType(val, typ1), typ2);
            }
            catch (Exception) { }
            return default(U);
        }
        [System.Diagnostics.DebuggerHidden()]
        private U TryChangeType<U, V>(V val)
            where U : IConvertible
        {
            try
            {
                Type typ1 = typeof(V), typ2 = typeof(U);
                return (U)Convert.ChangeType(Convert.ChangeType(val, typ1), typ2);
            }
            catch (Exception) { }
            return default(U);
        }
        #endregion
        #region "De- and Contructor"
        [System.Diagnostics.DebuggerHidden()]
        public CryptoRnd()
        {
            this._min = 0;
            this._max = 100;
            double d = nextRnd();
            for (Int32 i = 0; i <= 100; i++)
                d = this.nextRnd();
        }
        //[System.Diagnostics.DebuggerHidden()]
        //public ~CryptoRnd()
        //{
        //    GC.Collect();
        //    this.rng.Dispose();
        //}
        #endregion
    }
}

